import { aws_apigateway } from 'aws-cdk-lib';
import { Construct } from 'constructs';
import { OpenAPI3, OperationObject } from 'openapi-typescript';
import { BaseApi, BaseApiProps } from './base-api';
import { LambdaFunction, LambdaOptions } from './func';
export interface RestApiProps<OPS> extends BaseApiProps {
    /**
     * Generate routes for all endpoints configured in the openapi.yaml file
     *
     * @default true
     */
    autoGenerateRoutes?: boolean;
    /**
     * custom options for the created HttpApi
     *
     * @default -
     */
    restApiProps?: aws_apigateway.RestApiBaseProps;
    /**
     * additional options for the underlying Lambda function construct per operationId
     *
     * @default -
     */
    lambdaOptionsByOperation?: {
        [operationId in keyof OPS]?: LambdaOptions;
    };
    definitionFileName: string;
    cors: boolean;
}
/**
 * The RestApi construct sets up an AWS API Gateway REST API using OpenAPI specification.
 * This construct facilitates the creation of a REST API with various configurations, including custom domain, CORS support, and integration with Lambda functions.
 * It allows auto-generating routes based on the provided OpenAPI definition and provides methods to dynamically add custom routes and manage Lambda function integrations.
 *
 * @template PATHS - The type definition for the API paths.
 * @template OPS - The type definition for the API operations.
 *
 * @example
 * const api = new RestApi(this, 'MyRestApi', {
 *   apiName: 'MyAPI',
 *   stageName: 'dev',
 *   definitionFileName: 'openapi.yaml',
 *   authentication: myCognitoAuth,
 *   singleTableDatastore: myDynamoDBTable,
 *   autoGenerateRoutes: true,
 * });
 *
 * // Add a custom REST resource
 * api.addRestResource('/items', 'get');
 *
 * // Get the Lambda function for a specific operation
 * const lambdaFunction = api.getFunctionForOperation('getItems');
 */
export declare class RestApi<PATHS, OPS> extends BaseApi {
    private props;
    /**
     * The AWS API Gateway REST API instance.
     */
    readonly api: aws_apigateway.SpecRestApi;
    /**
     * The OpenAPI specification for the REST API.
     */
    readonly apiSpec: OpenAPI3;
    /**
     * A collection of Lambda functions used as integrations for the API operations.
     * @private
     */
    private _functions;
    /**
     * Creates an instance of RestApi.
     *
     * @param scope - The scope in which this construct is defined.
     * @param id - The scoped construct ID.
     * @param props - The properties of the RestApi construct.
     */
    constructor(scope: Construct, id: string, props: RestApiProps<OPS>);
    protected cleanupSpec(spec: {
        [key: string]: any;
    }): {
        [key: string]: any;
    };
    /**
     * return the generated Lambda function for the specified API operation
     */
    getFunctionForOperation(operationId: keyof OPS): LambdaFunction;
    /**
     * return a list of all generated Lambda functions
     */
    getFunctions(): LambdaFunction[];
    /**
     * Visitor method to modify the given functions
     *
     * @param operationIds the list of functions to visit
     * @param op the function to call for every function
     */
    modifyOperationFunctions(operationIds: (keyof OPS)[], op: (fn: LambdaFunction) => void): void;
    addRestResource<P extends keyof PATHS>(path: P, method: keyof PATHS[P]): LambdaFunction | undefined;
    addCustomRestResource(operation: OperationObject, method: string, description: string, additionalLambdaOptions?: LambdaOptions): LambdaFunction | undefined;
    private tableWriteAccessForMethod;
    /**
     * AWS does not properly apply the 'security' option to every single path-method.
     * According to documentation, global security gets applied, if there is no security
     * set for the particular path-method. If there is any, even empty, it will get precedence.
     * THIS DOES NOT HAPPEN ON AWS. IT JUST ALWAYS USES THE GLOBAL SETTING!
     *
     * 'security applies [...] schemes globally to all API operations, unless overridden on the operation level'
     * @see https://swagger.io/docs/specification/authentication/
     */
    protected patchSecurity(spec: OpenAPI3): void;
}
